@@ -1,11 +1,18 @@
+use strict;
+use warnings;
+
 package URI::Namespace;
-use Moose;
-use Moose::Util::TypeConstraints;
+use Moo 1.006000;
 use URI;
+use IRI 0.003;
+use Types::Namespace 0.004 qw( Iri );
+use namespace::autoclean;
+
+our $VERSION = '0.12';
 
 =head1 NAME
 
-URI::Namespace - A namespace URI class with autoload methods
+URI::Namespace - A namespace URI/IRI class with autoload methods
 
 
 =head1 SYNOPSIS
@@ -15,11 +22,9 @@ URI::Namespace - A namespace URI class with autoload methods
   print $foaf->as_string;
   print $foaf->name;
 
-
-
 =head1 DESCRIPTION
 
-This module provides an object with a URI attribute, typically used
+This module provides an object with a URI/IRI attribute, typically used
 prefix-namespace pairs, typically used in XML, RDF serializations,
 etc. The local part can be used as a method, these are autoloaded.
 
@@ -27,16 +32,22 @@ etc. The local part can be used as a method, these are autoloaded.
 
 =over
 
-=item C<< new ( $string | URI ) >>
+=item C<< new ( $string | URI | IRI ) >>
 
 This is the constructor. You may pass a string with a URI or a URI object.
 
 =item C<< uri ( [ $local_part ] ) >>
 
-Returns a L<URI> object with the namespace URI. Optionally, the method
+Returns a L<URI> object with the namespace IRI. Optionally, the method
 can take a local part as argument, in which case, it will return the
 namespace URI with the local part appended.
 
+=item C<< iri ( [ $local_part ] ) >>
+
+Returns a L<IRI> object with the namespace IRI. Optionally, the method
+can take a local part as argument, in which case, it will return the
+namespace IRI with the local part appended.
+
 =back
 
 The following methods from L<URI> can be used on an URI::Namespace object: C<as_string>, C<as_iri>, C<canonical>, C<eq>, C<abs>, C<rel>.
@@ -61,35 +72,46 @@ around BUILDARGS => sub {
     return { _uri => $parameters[0] };
 };
 
-class_type 'URI';
-coerce 'URI' => from 'Str' => via { URI->new($_) };
-
-has _uri => ( 
-    isa => 'URI',
-	 coerce => 1,
+has _uri => (
+    is => "ro",
+    isa => Iri,
+    coerce => 1,
     required => 1,
-	 reader => '_uri',
-	 handles => ['as_string', 'as_iri', 'canonical', 'eq', 'abs', 'rel']
+    handles => {
+        'as_string' => 'as_string',
+        'as_iri' => 'as_string',
+    }
 );
 
+sub iri {
+    my ($self, $name) = @_;
+    if (defined($name)) {
+        return IRI->new($self->_uri->as_string . "$name");
+    } else {
+        return $self->_uri;
+    }
+}
+
 sub uri {
     my ($self, $name) = @_;
-    return URI->new($self->_uri . "$name");
+    my $iri = $self->_uri->as_string;
+    if (defined($name)) {
+        return URI->new($iri . "$name");
+    } else {
+        return URI->new($iri);
+    }
+}
+
+for my $method (qw/ abs rel eq canonical /) {
+    eval qq[ sub $method { shift->uri->${method}(\@_) } ];
 }
 
 our $AUTOLOAD;
 sub AUTOLOAD {
-  my $self = shift;
-  my ($name) = $AUTOLOAD =~ /::(\w+)$/;
-  return $self->uri($name);
+    my $self = shift;
+    my ($name) = $AUTOLOAD =~ /::(\w+)$/;
+    return $self->uri($name);
 }
 
-
-
-no Moose::Util::TypeConstraints;
-no Moose;
-
-__PACKAGE__->meta->make_immutable();
-
 1;
 __END__
